/**
 * 上传设备固件版本步骤:
 *     1. 登录Control Hub接口（/api/login），获取 login cookie。
 *     2. 调用上传设备固件接口（/api/firmware/upload），上传设备固件到Control Hub。
 *     3. 调用应用固件接口（/api/firmware/add），固件才能够使用。
 *
 * Upload the device firmware steps:
 *     1. Use '/api/login' interface to log in. Obtain the Login cookie after logging in successfully.
 *     2. Use '/api/firmware/upload' interface to upload device firmware file(.mwf) to Control Hub.
 *     3. After the file(.mwf) is successfully uploaded, you need to use '/api/firmware/add' interface for the file(.mwf) to be valid.
 */
let HttpUtils = require('./httpUtils.js')
let httpUtils = new HttpUtils()

let constant = require('./constant.js')
const hubAddress = constant.CONTROL_HUB_ADDRESS
const userName = constant.USER_NAME
const password = constant.USER_PASSWORD

const uploadFirmwareURL = `http://${hubAddress}/api/firmware/upload`
const applyUploadFileURL = `http://${hubAddress}/api/firmware/add`
const uploadFilePath = './ultra_stream_hdmi_rev_c_1_2_68.mwf'

/**
 * 调用登录接口（/api/login），获取 login cookie。
 * Use '/api/login' interface to log in. Obtain the Login cookie after logging in successfully.
 */
httpUtils.reqStartLog(1, 'login request')
httpUtils.post(`http://${hubAddress}/api/login`,
    {
        params: { id: userName, pass: password },
        isJson: true
    })
    .then((response) => {
        httpUtils.log('Response', response)
        const resCookies = response['headers']['set-cookie']
        if (resCookies && resCookies.length > 0) {
            httpUtils.log('get login cookie', resCookies)
            // set request parameters
            let reqOpts = {
                headers: {
                    'Cookie': resCookies
                }
            }
            // upload firmware
            uploadFirmwareRequest(reqOpts)
        } else {
            httpUtils.log('get cookie failed', resCookies)
        }
    })
    .catch((err) => {
        httpUtils.log('Exception', err)
    })

/**
 * 调用上传设备固件接口（/api/firmware/upload），上传设备固件到 Control Hub。
 * Use '/api/firmware/upload' interface to upload device firmware file(.mwf) to Control Hub.
 */
const uploadFirmwareRequest = function (reqOpts) {
    httpUtils.reqStartLog(2, 'upload firmware request')
    httpUtils.upload(uploadFirmwareURL, uploadFilePath, reqOpts)
        .then((res) => {
            httpUtils.log('Response', res)
            applyUploadFileRequest(reqOpts, res['file-path'])
        })
        .catch((err) => {
            httpUtils.log('Exception', err)
        })
}

/**
 * 文件成功上传到 Control Hub 后，还需要调用应用固件接口（/api/firmware/add），固件才能够使用。
 * After the device firmware file(.mwf) is successfully uploaded to the cloud, you need to use this interface for the file(.mwf) to be valid.
 */
const applyUploadFileRequest = function (reqOpts, filePath) {
    httpUtils.reqStartLog(3, '/api/firmware/add request')

    reqOpts.headers['Content-Type'] = 'application/json; charset=UTF-8'
    httpUtils.post(applyUploadFileURL, { params: { 'file-path': filePath }, isJson: true, headers: reqOpts.headers })
        .then((res) => {
            const data = res.data || {}
            const isSuc = data.result === 0
            httpUtils.log('Response', isSuc ? 'Success!!!' : data)
        })
        .catch((err) => {
            httpUtils.log('Exception', err)
        })
}
